# Generated by roxygen2: do not edit by hand

S3method(loo,disbayes)
S3method(plot,disbayes)
S3method(plot,disbayes_hier)
S3method(print,disbayes)
S3method(summary,disbayes)
S3method(tidy,disbayes)
S3method(tidy,disbayes_hier)
export(ci2num)
export(conflict_disbayes)
export(disbayes)
export(disbayes_hier)
export(loo)
export(loo_indiv)
export(looi_disbayes)
export(plotfit_data_disbayes)
export(plotfit_disbayes)
export(tidy)
export(tidy_obsdat)
import(Rcpp)
import(dplyr)
import(methods)
import(tibble)
importFrom(generics,tidy)
importFrom(loo,loo)
importFrom(magrittr,"%>%")
importFrom(matrixStats,colQuantiles)
importFrom(mgcv,jagam)
importFrom(mgcv,s)
importFrom(mgcv,smoothCon)
importFrom(rstan,sampling)
importFrom(stats,coef)
importFrom(stats,dbinom)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pgamma)
importFrom(stats,qbeta)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(tidyr,extract)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
useDynLib(disbayes, .registration = TRUE)
